package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ProductType;

/**
 * Represents a signature for a {@link ConstructorOrOperation} and consolidates its attributes name, parameters,
 * containingType.
 */
public class ConstructorOrOperationSignature {
	private final String name;
	/**
	 * The parameters of this {@link ConstructorOrOperation}.
	 */
	private final ProductType parameters;
	/**
	 * The containingType is the name of the Class, that contains this constructor or operation.
	 */
	private final ClassType containingType;
	
	/**
	 * @param name
	 *            of the assigned {@link ConstructorOrOperation}
	 * @param parameters
	 *            of the assigned {@link ConstructorOrOperation}
	 * @param containingType
	 *            of the assigned {@link ConstructorOrOperation}
	 */
	private ConstructorOrOperationSignature(final String name,
			final ProductType parameters,
			final ClassType containingType) {
		this.name = name;
		this.parameters = parameters;
		this.containingType = containingType;
	}
	
	/**
	 * @param name
	 *            of the assigned {@link ConstructorOrOperation}
	 * @param parameters
	 *            of the assigned {@link ConstructorOrOperation}
	 * @param containingType
	 *            of the assigned {@link ConstructorOrOperation}
	 * @return Returns a new instance of a signature with the given arguments
	 */
	public static ConstructorOrOperationSignature create(final String name,
			final ProductType parameters,
			final ClassType containingType) {
		return new ConstructorOrOperationSignature(name, parameters, containingType);
	}
	
	@Override
	public int hashCode() {
		return this.getName().hashCode();
	}
	
	/**
	 * @param object
	 *            is the object to compare with "this"
	 * @return if and only if all properties given in the signature are equals this function returns true
	 */
	@Override
	public boolean equals(final Object object) {
		if (!(object instanceof ConstructorOrOperationSignature)) {
			return false;
		}
		final ConstructorOrOperationSignature signature = (ConstructorOrOperationSignature) object;
		if (!(this.name.equals(signature.name))) {
			return false;
		}
		if (!(this.containingType.equals(signature.containingType))) {
			return false;
		}
		if (!(this.parameters.equals(signature.parameters))) {
			return false;
		}
		return true;
	}
	
	/**
	 * @return the name of the assigned {@link ConstructorOrOperation}
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * @return the parameter of assigned {@link ConstructorOrOperation}
	 */
	public ProductType getParameters() {
		return this.parameters;
	}
	
	/**
	 * @return the containingType of the assigned {@link ConstructorOrOperation}
	 */
	public ClassType getContainingType() {
		return this.containingType;
	}
}
