package de.fhdw.wtf.common.ast.type;

import de.fhdw.wtf.common.ast.SyntaxObject;
import de.fhdw.wtf.common.constants.parser.AstDescriptionConstants;
import de.fhdw.wtf.common.token.Token;

/**
 * This class represents a component of a {@link ProductType}. The {@link ProductElementType} has a name and a
 * {@link Type}.
 */
public final class ProductElementType extends SyntaxObject implements Comparable<ProductElementType> {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -3322053592814738183L;
	
	/**
	 * Name of the Element.
	 */
	private final String name;
	
	/**
	 * Type of the Element.
	 */
	private final Type type;
	
	/**
	 * Constructor of {@link ProductElementType}.
	 * 
	 * @param name
	 *            Name of the Element.
	 * @param type
	 *            Type of the Element.
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	private ProductElementType(final String name, final Type type, final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
		this.name = name;
		this.type = type;
	}
	
	/**
	 * Creates a {@link ProductElementType}-Object.
	 * 
	 * @param name
	 *            Name of the Element.
	 * @param type
	 *            Type of the Element.
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @return the {@link ProductElementType}-Object
	 */
	public static ProductElementType create(final String name,
			final Type type,
			final Token firstToken,
			final Token lastToken) {
		return new ProductElementType(name, type, firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link ProductElementType}-Object.
	 * 
	 * @param name
	 *            Name of the Element.
	 * @param type
	 *            Type of the Element.
	 * @param firstToken
	 *            firstToken
	 * @return the {@link ProductElementType}-Object
	 */
	public static ProductElementType create(final String name, final Type type, final Token firstToken) {
		return new ProductElementType(name, type, firstToken, null);
	}
	
	@Override
	public int compareTo(final ProductElementType o) {
		final int typeCompare = o.getType().compareTo(this.getType());
		if (typeCompare == 0) {
			return this.name.compareTo(o.getName());
		}
		return typeCompare;
	}
	
	@Override
	public boolean equals(final Object o) {
		if (o instanceof ProductElementType) {
			final ProductElementType other = (ProductElementType) o;
			return this.getName().equals(other.getName()) && this.getType().equals(other.getType());
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return this.getName().hashCode() ^ this.getType().hashCode();
	}
	
	@Override
	public String toString() {
		final StringBuilder result = new StringBuilder();
		result.append(this.getName()).append(AstDescriptionConstants.COLON_TOKEN).append(this.getType());
		return result.toString();
	}
	
	/**
	 * Returns a string representation for this product element. See
	 * {@link de.fhdw.wtf.common.ast.type.Type#getTypeString()}.
	 * 
	 * @return A string representation for this product element.
	 */
	public String getTypeString() {
		final StringBuilder result = new StringBuilder();
		result.append(this.getName()).append(AstDescriptionConstants.COLON_TOKEN)
				.append(this.getType().getTypeString());
		return result.toString();
	}
	
	/**
	 * Name of the Element.
	 * 
	 * @return name
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * Type of the Element.
	 * 
	 * @return type
	 */
	public Type getType() {
		return this.type;
	}
	
}
