package de.fhdw.wtf.common.ast.visitor;

import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.ThrownType;

/**
 * Exception-Visitor for {@link de.fhdw.wtf.common.ast.type.CompositeType}.
 * 
 * @param <Y>
 */
public interface CompositeTypeVisitorException<Y extends Exception> {
	
	/**
	 * Handle-Method.
	 * 
	 * @param map
	 *            {@link MapType}
	 * @throws Y
	 *             Y
	 */
	void handle(MapType map) throws Y;
	
	/**
	 * Handle-Method.
	 * 
	 * @param list
	 *            {@link ListType}
	 * @throws Y
	 *             Y
	 */
	void handle(ListType list) throws Y;
	
	/**
	 * Handle-Method.
	 * 
	 * @param product
	 *            {@link ProductType}
	 * @throws Y
	 *             Y
	 */
	void handle(ProductType product) throws Y;
	
	/**
	 * Handle-Method.
	 * 
	 * @param sum
	 *            {@link SumType}
	 * @throws Y
	 *             Y
	 */
	void handle(SumType sum) throws Y;
	
	/**
	 * Handle-Method for a {@link ThrownType}-Object.
	 * 
	 * @param thrownType
	 *            the checked object
	 * @throws Y
	 *             Y
	 */
	void handle(ThrownType thrownType) throws Y;
}
