package de.fhdw.wtf.common.token.keywords;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the "operation"- or "function"-modifier as a token in WTF.
 */
public final class OperationToken extends KeywordToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -8890085544585927480L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 9728344;
	
	/**
	 * Defines constant "function" for {@link OperationToken}.
	 */
	private static final String TEXTUAL_REPRESENTATION = "function";
	
	/**
	 * Private constructor for {@link OperationToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the {@link KeywordToken} <function> in the original input.
	 */
	private OperationToken(final Position position) {
		super(position);
	}
	
	/**
	 * Factory method for {@link OperationToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link KeywordToken} in the original input.
	 * @return a new instance of this {@link KeywordToken}.
	 */
	public static OperationToken create(final Position position) {
		return new OperationToken(position);
	}
	
	@Override
	public boolean isOperationToken() {
		return true;
	}
	
	@Override
	public int getLength() {
		return OperationToken.TEXTUAL_REPRESENTATION.length();
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isOperationToken();
	}
	
	@Override
	public String stringRepresentation() {
		return TEXTUAL_REPRESENTATION;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
