package de.fhdw.wtf.dsl.scanner.modelScanner.state;

import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.CommentTextToken;
import de.fhdw.wtf.common.token.Position;

/**
 * ScannerState for recognition of Comments.<br>
 * Comments are initiated using the {@link CommentState#COMMENT_OPEN} symbol <br>
 * and cosed using the {@link CommentState#COMMENT_CLOSE} symbol.<br>
 * The commenttext is not restricted in any way.
 * 
 * @author tajoa
 */
public final class CommentState extends MultiCharState {
	
	/**
	 * Constant for creation.
	 */
	private static final String INITIAL_COMMENT = "";
	
	/**
	 * Comment open char.
	 */
	public static final char COMMENT_OPEN = '#';
	
	/**
	 * Comment close char.
	 */
	public static final char COMMENT_CLOSE = '#';
	
	/**
	 * <b>PRIVATE</b> Constructor.
	 * 
	 * @param currentComment
	 *            initial comment
	 * @param firstPosition
	 *            of the first char.
	 */
	private CommentState(final String currentComment, final Position firstPosition) {
		super(currentComment, firstPosition);
	}
	
	@Override
	public void finish(final TokenStream outputStream) {
		outputStream.add(CommentTextToken.create(
				this.getCurrentInput().concat(Character.toString(CommentState.COMMENT_CLOSE)),
				this.getFirstPosition()));
	}
	
	/**
	 * Factory for {@link CommentState}.
	 * 
	 * @param firstPosition
	 *            of the first char.
	 * @return {@link CommentState}
	 */
	public static CommentState create(final Position firstPosition) {
		return new CommentState(INITIAL_COMMENT, firstPosition);
	}
	
	@Override
	public boolean accept(final ScannerStateVisitor visitor) {
		return visitor.handleCommentState();
	}
	
}
